package org.doit.muffin;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Properties;

/**
 * The OppNetCache class executes web caching tasks.
 *  
 * @author  Taotao Li <tl2453@columbia.edu>
 * @version 2011.0518
 */

public class OppnetHelper {
	
	private OppnetHelper(){
	}
	
	private static OppnetHelper oppnetHelper = null;
	private static String CardNameForInternetConnection=null;
	private static String CardNameForLANConnection=null;
	private static String CATALINA_HOME_KEY = "Catalina_Home";
	private static String CATALINA_HOME_VALUE = null;
	private static String TOMCAT_PORT_KEY = "Tomcat_Port_Num";
	private static String TOMCAT_PORT_VALUE=null;
	private static String ESSID_KEY = "ESSID";
	private static String ESSID_VALUE=null;
	
	
	public static OppnetHelper getInstance(){
		if(oppnetHelper==null){
			oppnetHelper = new OppnetHelper();
			loadOppnetConf();
		}
		return oppnetHelper;
	}
	
	public static void loadOppnetConf(){
		Properties p = new Properties();
		try {
			p.load(new FileReader("./oppnet.conf"));
			CATALINA_HOME_VALUE = (String)p.get(CATALINA_HOME_KEY);
			TOMCAT_PORT_VALUE = (String)p.get(TOMCAT_PORT_KEY);
			ESSID_VALUE = (String)p.get(ESSID_KEY);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public String getCatalinaHome(){
		return CATALINA_HOME_VALUE;
	}
	
	public String getTomcatPort(){
		return TOMCAT_PORT_VALUE;
	}
	
	private void analyzeNetworkCard(){
		try {
			Process command = Runtime.getRuntime().exec("iwconfig");
			BufferedReader in = new BufferedReader(new InputStreamReader(
					command.getInputStream()));
			String line;
			String targetLine;
			while( (line = in.readLine()) != null){
				if(line.contains("IEEE")){
					targetLine=line+in.readLine();
					String[] array = targetLine.split(" ");
					for(int i=0;i<array.length;i++){
						if(array[i].contains(ESSID_VALUE)){
							CardNameForLANConnection = array[0];
							System.out.println("Card for LAN:"+CardNameForLANConnection);
						}else if (array[i].contains("Managed")){
							CardNameForInternetConnection = array[0];
							System.out.println("Card for WWW:"+CardNameForInternetConnection);
						}
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private String getIpAddressByCardName(String name){
		String ip = null;
		if(name!=null){
			try {
				Enumeration<NetworkInterface> netInterfaces = NetworkInterface.getNetworkInterfaces();
				while (netInterfaces.hasMoreElements()) {  
	                 NetworkInterface ni = netInterfaces.nextElement();
	                 if(ni.getName().equals(name)){
	                	 Enumeration<InetAddress> address = ni.getInetAddresses();
	                	 while (address.hasMoreElements()) {  
		                     ip = address.nextElement().getHostAddress();
	                	 }
	                	 return ip;
	                 }
				}
			} catch (SocketException e) {
				e.printStackTrace();
			}  
			
		}
		return ip;
	}
	
	public String getIpAddressForLAN(){
		analyzeNetworkCard();
		return getIpAddressByCardName(CardNameForLANConnection);
	}
	 	
	public static void main(String[] arg){
		OppnetHelper.getInstance().analyzeNetworkCard();
		OppnetHelper.getInstance().getIpAddressByCardName(CardNameForLANConnection);
		OppnetHelper.getInstance().getIpAddressByCardName(CardNameForInternetConnection);
	}

}
