package org.doit.muffin;

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * The OppNetCache class executes web caching tasks.
 * 
 * @author Shuai Yue <sy2342@columbia.edu>
 * @version 2010.1216
 * 
 * @author Taotao Li <tl2453@columbia.edu>
 * @version 2011.0518
 */

public class OppNetCache extends Thread {
	private static String catalinaHome = OppnetHelper.getInstance()
			.getCatalinaHome();
	public static final String LOCALROOT = catalinaHome + "webapps/CacheFiles/";
	private static String tomcatPort = OppnetHelper.getInstance()
			.getTomcatPort();
	public static final String IpAddressFroLAN = OppnetHelper.getInstance()
			.getIpAddressForLAN();
	public static final String VISITPATH = "http://" + IpAddressFroLAN + ":"
			+ tomcatPort + "/CacheFiles/";
	ArrayList<String> urlList = new ArrayList<String>();
	ArrayList<String> allUrlList = new ArrayList<String>();
	ArrayList<String> subLink = new ArrayList<String>();
	boolean parseSuccess = true;
	boolean downloadSuccess = true;
	String initialUrl;

	OppNetCache(String str) {
		super(str);
	}

	public void run() {
		try {
			for (;;) {
				int listLength = Server.REQLIST.size();

				if (listLength != 0) {
					String queueArray[][] = new String[listLength][2];
					for (int i = 0; i < listLength; i++) {
						queueArray[i][0] = Server.REQLIST.get(i).toString();
						queueArray[i][1] = Server.NUMLIST.get(i).toString();
					}

					Arrays.sort(queueArray, new TwoDimArrayComparator());

					for (int i = 0; i < listLength; i++) {
						for (int j = 0; j < 2; j++) {
							System.out.print(queueArray[i][j] + "\t");
						}
						System.out.println();
					}

					System.out.println("The length of the queue is "
							+ listLength + "\n\n");
					if (isInternetReachable()) {

						int oldQueueSize = listLength;

						cache(queueArray[0][0], false);

						if (Server.REQLIST.size() == oldQueueSize) {
							cache(queueArray[0][0], false);

							if (Server.REQLIST.size() == oldQueueSize) {
								int deletePositon = Server.REQLIST
										.indexOf(queueArray[0][0]);

								if (deletePositon != -1) {
									Server.REQLIST.remove(deletePositon);
									Server.NUMLIST.remove(deletePositon);
								}

							}

						}

						System.out.println("The length of the queue is "
								+ Server.REQLIST.size() + "\n");

					} else {

						System.out
								.println("Internet is currently unreachable. Please wait.");
						Thread.sleep(500);
					}
				} else {

					if (subLink.size() != 0) {

						if (isInternetReachable()) {
							int subOldSize = subLink.size();

							cache(subLink.get(0), true);

							if (subLink.size() == subOldSize) {
								subLink.remove(0);
							}

						} else {

							System.out
									.println("Internet is currently unreachable. Pleasw wait.");
							Thread.sleep(500);
						}
					} else
						System.out.println("There is no request in the queue.");

				}

			}

		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

	}

	public boolean isInternetReachable() {
		boolean reachable = false;
		try {
			InetAddress address = InetAddress.getByName("www.google.com");
		} catch (UnknownHostException e) {
			reachable = false;
		}
		reachable = true;

		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {

			e.printStackTrace();
		}
		return reachable;
	}

	void cache(String ini, boolean isSubLink) {

		long start = System.currentTimeMillis();

		initialUrl = ini;
		parseSuccess = true;
		downloadSuccess = true;

		if (initialUrl.endsWith("/")) {
			initialUrl = initialUrl.substring(0, initialUrl.length() - 1);
		}

		importURLLIST();

		parseURL(initialUrl, isSubLink);

		if (parseSuccess && !urlList.isEmpty()) {

			for (int i = 0; i < urlList.size(); i++) {
				downloadContent(urlList.get(i));
				if (!downloadSuccess) {
					break;
				}
			}

			if (downloadSuccess) {

				long end = System.currentTimeMillis();
				double seconds = (end - start) / 1000.0;

				if (!urlList.contains(initialUrl)) {
					urlList.add(initialUrl);
				}

				createCacheTable();

				createWebsiteTable();

				urlList.clear();
				allUrlList.clear();
				System.out.println("\n" + initialUrl
						+ " is cached successfully in " + seconds + " seconds");

				if (!isSubLink) {

					int deletePositon = Server.REQLIST.indexOf(initialUrl);
					if (deletePositon != -1) {
						Server.REQLIST.remove(deletePositon);
						Server.NUMLIST.remove(deletePositon);
					}
				} else {

					subLink.remove(0);
				}
			} else {

				urlList.clear();
				allUrlList.clear();

				System.out.println("\nFailed to cache " + initialUrl);
			}

		} else {
			System.out.println("\nFailed to cache " + initialUrl);
		}
	}

	void importURLLIST() {
		try {
			FileReader fr = new FileReader(LOCALROOT + "CacheTable.txt");
			BufferedReader br = new BufferedReader(fr);
			String line = new String();

			while ((line = br.readLine()) != null) {
				allUrlList.add(line);
			}
			br.close();
			fr.close();
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

	}

	void parseURL(String initialUrl, boolean isSub) {
		try {
			URL u = new URL(initialUrl);
			URLConnection uc = u.openConnection();
			uc.setReadTimeout(2000);
			BufferedInputStream in = new BufferedInputStream(uc
					.getInputStream());

			String type = uc.getContentType().toLowerCase();
			int contentLength = 0;

			if (uc.getContentLength() != -1) {
				contentLength = uc.getContentLength();
			} else {
				contentLength = 8192;
			}

			if (type.startsWith("text/html")) {
				File cache = null;
				String host = u.getHost();
				String path = u.getPath();

				if (path.equals("")) {
					cache = new File(LOCALROOT + host + ".html");
				} else if (path.endsWith(".html")) {
					cache = new File(LOCALROOT + host + "/" + path);
				} else {
					cache = new File(LOCALROOT + host + "/" + path + ".html");
				}

				cache.getParentFile().mkdirs();
				PrintWriter pw = new PrintWriter(new FileOutputStream(cache));
				byte b[] = new byte[contentLength];
				int numRead = in.read(b);
				String content = new String(b, 0, numRead);

				while (numRead != -1) {
					numRead = in.read(b);

					if (numRead != -1) {
						String newContent = new String(b, 0, numRead);
						content += newContent;
					}
				}

				in.close();

				String addedContent = "";
				int indexBegin = content.indexOf("</head>");

				indexBegin = indexBegin + 7;
				addedContent = content.substring(0, indexBegin)
						+ "<iframe src =\"http://"
						+ IpAddressFroLAN
						+ ":"
						+ tomcatPort
						+ "/StatusBar/statusICE.iface\" width=\"100%\" height=\"40\" scrolling=\"auto\" style=\"background-color: #EEEEEE\">"
						+ "<p>Your browser does not support iframes.</p></iframe>"
						+ content.substring(indexBegin, content.length());
				content = addedContent;

				String lowerCaseContent = content.toLowerCase();
				String modifiedContent = "";
				String modifiedUrl = "";
				int indexFront = 0;
				int indexBack = 0;

				while ((indexFront = lowerCaseContent.indexOf("src=\"",
						indexFront)) != -1) {
					indexFront = indexFront + 5;

					modifiedContent = modifiedContent
							+ content.substring(indexBack, indexFront);

					indexBack = lowerCaseContent.indexOf("\"", indexFront);
					String contentUrl = content
							.substring(indexFront, indexBack);

					if (!contentUrl.contains("://")) {
						if (contentUrl.startsWith("/")) {
							contentUrl = initialUrl + contentUrl;
						} else if (contentUrl.startsWith("../")) {
							int length = contentUrl.length();
							contentUrl = initialUrl
									+ contentUrl.substring(2, length);
						} else
							contentUrl = initialUrl + "/" + contentUrl;
					}

					if (contentUrl.contains("?")) {
						int qMark = contentUrl.indexOf("?");
						contentUrl = contentUrl.substring(0, qMark);
					}

					if (contentUrl.endsWith("/")) {
						contentUrl = contentUrl.substring(0, contentUrl
								.length() - 1);
					}

					boolean belongType = contentUrl.endsWith(".css")
							|| contentUrl.endsWith(".js")
							|| contentUrl.endsWith(".xml")
							|| contentUrl.endsWith(".html")
							|| contentUrl.endsWith(".ico")
							|| contentUrl.endsWith(".bmp")
							|| contentUrl.endsWith(".gif")
							|| contentUrl.endsWith(".jpg")
							|| contentUrl.endsWith(".jpeg")
							|| contentUrl.endsWith(".tif")
							|| contentUrl.endsWith(".png")
							|| contentUrl.endsWith(".tiff")
							|| contentUrl.endsWith(".svg")
							|| contentUrl.endsWith(".swf")
							|| contentUrl.endsWith(".CSS")
							|| contentUrl.endsWith(".JS")
							|| contentUrl.endsWith(".XML")
							|| contentUrl.endsWith(".HTML")
							|| contentUrl.endsWith(".ICO")
							|| contentUrl.endsWith(".BMP")
							|| contentUrl.endsWith(".GIF")
							|| contentUrl.endsWith(".JPG")
							|| contentUrl.endsWith(".JPEG")
							|| contentUrl.endsWith(".TIF")
							|| contentUrl.endsWith(".PNG")
							|| contentUrl.endsWith(".TIFF")
							|| contentUrl.endsWith(".SVG")
							|| contentUrl.endsWith(".SWF");

					if (!contentUrl.startsWith("http://")
							|| contentUrl.contains(";")
							|| contentUrl.contains("'")
							|| contentUrl.contains("\"")
							|| contentUrl.contains("(")
							|| contentUrl.contains(")") || !belongType) {

						modifiedContent = modifiedContent + contentUrl;
						continue;
					}

					boolean alreadyExist = urlList.contains(contentUrl)
							|| allUrlList.contains(contentUrl);
					boolean isIniUrl = contentUrl.equalsIgnoreCase(initialUrl);

					if ((!alreadyExist) && (!isIniUrl)) {
						urlList.add(contentUrl);

					}

					modifiedUrl = contentUrl.replace("http://", VISITPATH);
					modifiedContent = modifiedContent + modifiedUrl;
				}

				int lastIndex = content.length();
				modifiedContent = modifiedContent
						+ content.substring(indexBack, lastIndex);

				String lowerCaseContent1 = modifiedContent.toLowerCase();
				String modifiedContent1 = "";
				String modifiedUrl1 = "";
				int indexFront1 = 0;
				int indexBack1 = 0;

				while ((indexFront1 = lowerCaseContent1.indexOf("href=\"",
						indexFront1)) != -1) {
					indexFront1 = indexFront1 + 6;
					modifiedContent1 = modifiedContent1
							+ modifiedContent
									.substring(indexBack1, indexFront1);
					indexBack1 = lowerCaseContent1.indexOf("\"", indexFront1);
					String contentUrl1 = modifiedContent.substring(indexFront1,
							indexBack1);

					if (!contentUrl1.contains("://")) {
						if (contentUrl1.startsWith("/")) {
							contentUrl1 = initialUrl + contentUrl1;
						} else if (contentUrl1.startsWith("../")) {
							int length = contentUrl1.length();
							contentUrl1 = initialUrl
									+ contentUrl1.substring(2, length);
						} else
							contentUrl1 = initialUrl + "/" + contentUrl1;
					}

					if (contentUrl1.contains("?")) {
						int qMark = contentUrl1.indexOf("?");
						contentUrl1 = contentUrl1.substring(0, qMark);
					}

					if (contentUrl1.endsWith("/")) {
						contentUrl1 = contentUrl1.substring(0, contentUrl1
								.length() - 1);
					}

					if (contentUrl1.endsWith(".css")
							|| contentUrl1.endsWith(".ico")) {

						boolean alreadyExist = urlList.contains(contentUrl1)
								|| allUrlList.contains(contentUrl1);
						boolean isIniUrl = contentUrl1
								.equalsIgnoreCase(initialUrl);

						if ((!alreadyExist) && (!isIniUrl)) {
							urlList.add(contentUrl1);

						}

						modifiedUrl1 = contentUrl1
								.replace("http://", VISITPATH);
						modifiedContent1 = modifiedContent1 + modifiedUrl1;
					}

					else if (contentUrl1.startsWith(initialUrl)
							&& !contentUrl1.contains(";")
							&& !contentUrl1.contains("'")
							&& !contentUrl1.contains("\"")
							&& !contentUrl1.contains("(")
							&& !contentUrl1.contains(")")
							&& !contentUrl1.contains(".php")
							&& contentUrl1.endsWith(".html") && !isSub) {

						boolean alreadyExist = urlList.contains(contentUrl1)
								|| allUrlList.contains(contentUrl1);
						boolean isIniUrl = contentUrl1
								.equalsIgnoreCase(initialUrl);

						if ((!alreadyExist) && (!isIniUrl)) {
							subLink.add(contentUrl1);
						}

						modifiedContent1 = modifiedContent1 + contentUrl1;
					}

					else {

						modifiedContent1 = modifiedContent1 + contentUrl1;
					}

				}

				int lastIndex1 = modifiedContent.length();
				modifiedContent1 = modifiedContent1
						+ modifiedContent.substring(indexBack1, lastIndex1);

				pw.println(modifiedContent1);
				pw.close();

			} else {

				urlList.add(initialUrl);
			}

		} catch (Exception e) {
			System.err.println(e.getMessage());
			if (e.getMessage().startsWith("No route to host")
					|| e.getMessage().contains("time"))

				parseSuccess = false;
		}
	}

	void downloadContent(String u) {
		try {

			URL url = new URL(u);

			URLConnection uc = url.openConnection();
			uc.setReadTimeout(2000);
			BufferedInputStream in = new BufferedInputStream(uc
					.getInputStream());
			String type = uc.getContentType().toLowerCase();
			int contentLength = 0;

			if (uc.getContentLength() != -1) {
				contentLength = uc.getContentLength();
			} else {
				contentLength = 8192;
			}

			String host = url.getHost();
			String path = url.getPath();
			File cache = null;

			if (type.startsWith("text")) {
				cache = new File(LOCALROOT + host + "/" + path);
				cache.getParentFile().mkdirs();

				PrintWriter pw = new PrintWriter(new FileOutputStream(cache));
				byte b[] = new byte[contentLength];
				int numRead = in.read(b);

				if (numRead >= 0) {
					String content = new String(b, 0, numRead);
					while (numRead != -1) {
						numRead = in.read(b);
						if (numRead != -1) {
							String newContent = new String(b, 0, numRead);
							content += newContent;
						}
					}

					in.close();
					pw.println(content);
					pw.close();

					allUrlList.add(u);
				}

			}

			else {
				cache = new File(LOCALROOT + host + "/" + path);

				cache.getParentFile().mkdirs();
				byte[] b = new byte[contentLength];
				int bytesRead = 0;
				int offset = 0;

				while (offset < contentLength) {
					bytesRead = in.read(b, offset, b.length - offset);
					if (bytesRead == -1)
						break;
					offset += bytesRead;
				}
				in.close();

				FileOutputStream out = new FileOutputStream(cache);
				out.write(b);
				out.flush();
				out.close();

				allUrlList.add(u);

			}

		} catch (Exception e) {
			System.err.println(e.getMessage());
			if (e.getMessage().startsWith("No route to host")
					|| e.getMessage().contains("time"))

				downloadSuccess = false;
		}
	}

	void createCacheTable() {
		try {
			FileWriter fw = new FileWriter(LOCALROOT + "CacheTable.txt", true);
			PrintWriter out = new PrintWriter(fw);

			int listSize = urlList.size();

			for (int j = 0; j < listSize; j++) {
				String listStr = urlList.get(j).toString();
				out.println(listStr);
			}

			out.close();
			fw.close();

		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

	void createWebsiteTable() {
		try {
			FileWriter fw = new FileWriter(LOCALROOT + "GuidePage.html", true);
			PrintWriter out = new PrintWriter(fw);

			String str = initialUrl;
			str = str.substring(7, str.length());

			if (!str.contains("/")) {
				if (str.endsWith("html")) {
					str = "<a href=\"http://" + IpAddressFroLAN + ":"
							+ tomcatPort + "/CacheFiles/" + str + "\">" + str
							+ "</a>" + "<br/>";
				} else {
					str = "<a href=\"http://" + IpAddressFroLAN + ":"
							+ tomcatPort + "/CacheFiles/" + str + ".html"
							+ "\">" + str + "</a>" + "<br/>";
				}

				out.println(str);
			}

			out.close();
			fw.close();

		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
}

class TwoDimArrayComparator implements Comparator {

	public TwoDimArrayComparator() {
	}

	public int compare(Object o1, Object o2) {

		String row1[] = (String[]) o1;
		String row2[] = (String[]) o2;
		return (Integer.parseInt(row2[1]) - Integer.parseInt(row1[1]));

	}
}
