package org.doit.muffin.filter;

import org.doit.muffin.*;

import java.util.regex.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * The OppNetFilter class processes the request, check cache and add new items
 * into queue.
 * 
 * @author Shuai Yue <sy2342@columbia.edu>
 * @version 2010.1216
 * 
 * @author Taotao Li <tl2453@columbia.edu>
 * @version 2011.0518
 */

public class OppNetFilter implements RequestFilter {
	Prefs prefs;
	OppNet factory;
	private static String catalinaHome = OppnetHelper.getInstance()
			.getCatalinaHome();
	private static String tomcatPort = OppnetHelper.getInstance()
			.getTomcatPort();
	public static final String IpAddressFroLAN = OppnetHelper.getInstance()
			.getIpAddressForLAN();
	public static final String LOCALROOT = catalinaHome + "webapps/CacheFiles/";

	OppNetFilter(OppNet factory) {
		this.factory = factory;
	}

	public void setPrefs(Prefs prefs) {
		this.prefs = prefs;
	}

	public void filter(Request request) throws FilterException {
		if (!request.getURL().startsWith("http://" + IpAddressFroLAN)) {
			try {

				String urlStr = request.getURL();
				URL url = new URL(urlStr);
				String host = url.getHost();
				String path = url.getPath();

				if (urlStr.contains("?")) {
					int qMark = urlStr.indexOf("?");
					urlStr = urlStr.substring(0, qMark);
				}

				boolean flag = false;
				if (path.length() == 1 && urlStr.endsWith("/")) {
					urlStr = urlStr.substring(0, urlStr.length() - 1);
					flag = true;
				}
				if (path.length() > 1 && urlStr.endsWith("/")) {
					urlStr = urlStr.substring(0, urlStr.length() - 1) + ".html";
				}

				if (IsBrowserSentRequest(urlStr)) {
					request.setURL("http://" + IpAddressFroLAN + ":"
							+ tomcatPort + "/CacheFiles/InvalidPage.html");// need
					// to
					// improve
				} else if (IsNonHTML(urlStr)) {
					request.setURL("http://" + IpAddressFroLAN + ":"
							+ tomcatPort + "/CacheFiles/InvalidPage.html");// need
					// to
					// improve
				} else {
					FileReader fr = new FileReader(LOCALROOT + "CacheTable.txt");
					BufferedReader br = new BufferedReader(fr);
					String line = new String();
					boolean inCache = false;

					while ((line = br.readLine()) != null) {
						if (urlStr.equals(line)) {
							inCache = true;
						}
					}

					br.close();
					fr.close();

					if (inCache) {
						if (flag) {
							urlStr = urlStr + ".html";
						}
						urlStr = urlStr.replace("http://", "http://"
								+ IpAddressFroLAN + ":" + tomcatPort
								+ "/CacheFiles/");
						request.setURL(urlStr);
					} else {
						request.setURL("http://" + IpAddressFroLAN + ":"
								+ tomcatPort + "/CacheFiles/"
								+ "GuidePage.html");// need to improve
						AddToWaitingList(urlStr);
						urlStr = urlStr.substring(7, urlStr.length());
						if (!urlStr.contains("/")) {
							AddToMappingTable(urlStr, request
									.getHeaderField("X-Client-IP"));
						}
					}
				}
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}
		}
	}

	public static void AddToWaitingList(String urlStr) {
		try {
			if (!Server.REQLIST.contains(urlStr)) {
				Server.REQLIST.add(urlStr);
				Server.NUMLIST.add(1);
			} else {
				Server.NUMLIST.set(Server.REQLIST.indexOf(urlStr),
						(Integer) Server.NUMLIST.get(Server.REQLIST
								.indexOf(urlStr)) + 1);
			}

		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

	public boolean IsBrowserSentRequest(String str) {
		if (!str.contains(".")
				|| str.contains("http://fxfeeds.mozilla.com")
				|| str.contains("addons.mozilla.org")
				|| str.endsWith("favicon.ico")
				|| str.contains("clients1.google.com")
				|| str.contains("clients2.google.com")
				|| str.contains("clients3.google.com")
				|| str.contains("clients4.google.com")
				|| str.contains("clients5.google.com")
				|| str.contains("tools.google.com")
				|| str.contains("http://safebrowsing.clients.google.com")
				|| str.contains("http://sitecheck2.opera.com")
				|| str
						.contains("http://alert.services.conduit.com/Alerts/AlertService.asmx")
				|| str.contains("http://l.yimg.com/a/combo")
				|| str.contains("b.scorecardresearch.com/b")) {
			return true;
		} else {
			return false;
		}
	}

	public boolean IsNonHTML(String str) {
		return false;
	}

	public void AddToMappingTable(String strUrl, String strIP) {
		try {
			FileWriter fw = new FileWriter(LOCALROOT + "MappingTable.txt", true);
			PrintWriter out = new PrintWriter(fw);
			String str = strUrl + "," + strIP;
			out.println(str);
			out.close();
			fw.close();
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

}
